@@ -117,9 +117,11 @@ texts = {
 操作其实很简单，一个基本的原则就是，任何走势，无论怎么折腾，都逃不出这个节奏，就是底、
 顶以及连接两者的中间过程，因此，在两头的操作节奏就是中枢震荡，只是底的时候要先买后卖，
 顶的时候要先卖后买，这样更安全点。至于中间的连接部分，就是持有，当然，对于空头走势，小
-板凳就是一个最好的持有，一直持有到底部构造完成。而有技术的，根本就不需要什么小板凳，按
-操作级别，分清楚目前是三阶段中的哪一段，然后日日是好日，时时是花时，不赚钱那真是脑子有
-水了。亏钱都是错误操作引起的，不断反省，才会有进步的。
+板凳就是一个最好的持有，一直持有到底部构造完成。
+
+而有技术的，根本就不需要什么小板凳，按操作级别，分清楚目前是三阶段中的哪一段，然后日日
+是好日，时时是花时，不赚钱那真是脑子有水了。亏钱都是错误操作引起的，不断反省，才会有进
+步的。
 
 --摘自《2008-08-29 09:15 教你炒股票108：何谓底部？从月线看中期走势演化》
 """,
@@ -743,6 +745,84 @@ ID要强调当下分解的多样性？因为走势本身就是当下形成中的
 一切都从也只能从现实出发，现实在什么阶段，什么位次，是必须首要分析的问题。
 
 --摘自《2006-11-10 12:00 《论语》详解：给所有曲解孔子的人（27）》
+""",
+
+    "85": """
+判断走势，如同中医看病，未病而治的是第一等的，次之的是对治欲病，到已病阶段，那只能算是
+亡羊补牢了。但绝大多数的人，病入膏肓了还在幻想，市场里最终牺牲的，总是这种人。
+
+--摘自《2007-12-17 21:40 教你炒股票91：走势结构的两重表里关系1》
+""",
+
+    "86": """
+划分的原则很明确，就是必须保证中枢的确立，在这前提下，可以根据结合律，使得连接中枢的走
+势保持最完美的形态。
+
+--摘自《2008-02-18 16:19 教你炒股票99：走势结构的两重表里关系3》
+""",
+
+    "87": """
+1. 善猎者善待，耐心观察，等待时机。
+2. 影响价格的终归价格本身，影响走势的终归走势本身，影响你的终归你自己。
+3. 莫问为何，接受走势。走势即因，走势即果，涨跌循环，因果往复。
+4. 先观察市场走势对消息的反馈，再寻找买卖点介入，切忌追高。
+5. 任何在没有出现背驰产生买点的抄底都是不理智的。
+6. 创新高只是区分前后两段走势，而不是操作标准。
+7. 你无法征服市场，但可以感受市场脉络，如掌中纹般清晰可见。
+8. 日内操作要细腻很多，难度更大，但对结果影响反而没那么重要，能力达不到莫要强求。
+9. 注意不要追高，一次追高也许会有甜头，次次追高就要尝苦头了。
+10. 保持耐心，小菜开胃，大餐饱腹。
+
+--摘自《即缠非缠》微博
+""",
+
+    "88": """
+11. 真正的猎手不是动动嘴皮子吹出来的，既能善看，又要敢干。
+12. 机会在恐慌蔓延中萌发,耐心观察。
+13. 不要着急抄底，等待背驰构成，出现买卖点再行操作。
+14. 股票、论语、易经等凡此种种皆为媒介，以媒介为引，悟道，悟禅，悟己。
+15. 任何级别的背驰亦都是从一分钟级别背驰发生发展而出。
+16. 莫因行情火爆而头脑发热，而是冷静客观分析走势，寻找自己能把握的买卖点操作。
+17. 要降低个人对消息的乐观预期，聚焦于当下行情走势本身，把握自己能把握的买卖点。
+18. 既不要恐高也不要怕跌，买点买卖点卖。
+19. 上下震荡是在所难免的，能否利用震荡来降低成本这也是一个分水岭。
+20. 不要对任何消息抱有过度的乐观和悲观，认真观察分析市场，市场走势会给出答案。
+21. 高开很容易就会把人的贪婪情绪调动起来。
+
+--摘自《即缠非缠》微博
+""",
+
+    "89": """
+底部都是分级别的，如果站在精确走势类型的角度，那么第一类买点出现后，直到该买点所引发
+的中枢第一次走出第三类买卖点前，都可以看成底部构造的过程。只不过如果是第三类卖点先出
+现，就意味着这底部构造失败了，反之，第三类买点意味着底部构造的最终完成并展开新的行情。
+当然，顶部的情况，反过来定义就是。
+
+站在分型的角度，底部就是构成底分型的那个区间，而跌破分型最低点意味着底部构成失败，反
+之，有效站住分型区间上边沿，就意味着底部构造成功并至少展开一笔一上行情。
+
+有了底部定义，就一定要搞明白，不是在底部的区间上买，而是相反，应该和中枢震荡的操作一
+样，在区间下探失败时买，这才是最好的买点。
+
+--摘自《2008-08-29 09:15 教你炒股票108：何谓底部？从月线看中期走势演化》
+""",
+
+    "90": """
+理论的探讨，是为了树立操作的信心，当然，还为了对走势有一个精确的分析去指导操作，但其心理
+层面的意义也是极为重要的。这绝对不能迷信，因为相信本ID而相信本ID的理论，那就是绝对的脑子
+进水了。而是要从道理、逻辑等方法彻底搞清楚，这样才能无疑地去操作，而不用瞻前顾后。
+
+--摘自《2007-04-27 08:42 教你炒股票50：操作中的一些细节问题》
+""",
+
+    "91": """
+心态要平稳点，不要整天去计算今天少挣多少诸如此类的问题，说白了，如果你没有一套有效的方法，
+只要你在市场里，你赚的钱从本质上就不是你的，只是暂时存在你那里。而要把自己培养成一个赚钱
+机器，就如同前锋把自己培养成射门机器一样，方法学了都会，但神射手却不一定都是，这需要更多
+的努力。市场的技术，是需要磨练的。关键是真正掌握技术，只要掌握了，赚钱就成了自然的事情，
+只要有足够的时间，就自然产生足够的钱。
+
+--摘自《2007-04-27 08:42 教你炒股票50：操作中的一些细节问题》
 """,
 }
 
