@@ -18,72 +18,18 @@ from sklearn.preprocessing import KBinsDiscretizer
 
 from .. import envs
 from ..traders.advanced import CzscAdvancedTrader, BarGenerator
-from ..traders.utils import freq_cn2ts
-from ..data.ts_cache import TsDataCache
+from ..data import TsDataCache, freq_cn2ts
 from ..objects import RawBar, Signal
 from ..utils.cache import home_path
 
 
-@deprecated(reason='请使用 SignalsPerformance 来分析信号表现', version="0.9.0")
-def get_dfs_base(dfs: pd.DataFrame):
-    """获取所有信号的基础信息"""
-    cols = [x for x in dfs.columns if x[0] == 'n' and x[-1] == 'b'] \
-           + [x for x in dfs.columns if x[0] == 'b' and x[-1] == 'b']
-    info = {"name": "基准", "count": len(dfs), "cover": 1.0}
-    info.update(dfs[cols].mean().to_dict())
-    dtm = {f"dt_{k}": v for k, v in dfs.groupby('dt')[cols].mean().mean().to_dict().items()}
-    info.update(dtm)
-    return info
-
-
-@deprecated(reason='请使用 SignalsPerformance 来分析信号表现', version="0.9.0")
-def analyze_signal_keys(dfs: pd.DataFrame, keys: List[str], mode: int = 0) -> pd.DataFrame:
-    """分析信号组合的分类能力
-
-    :param dfs: 信号表
-    :param keys: 信号组合
-    :param mode: 分析模式，0 同时分析时序和截面表现；1 时序表现；2 截面表现
-    :return:
-    """
-    len_dfs = len(dfs)
-    cols = [x for x in dfs.columns if x[0] == 'n' and x[-1] == 'b'] \
-           + [x for x in dfs.columns if x[0] == 'b' and x[-1] == 'b']
-
-    def __static(_df, _name):
-        _res = {"name": _name, "count": len(_df), "cover": round(len(_df) / len_dfs, 4)}
-        if mode in [0, 1]:
-            _r1 = _df[cols].mean().to_dict()
-        else:
-            _r1 = {}
-        if mode in [0, 2]:
-            _r2 = {f"dt_{k}": v for k, v in _df.groupby('dt')[cols].mean().mean().to_dict().items()}
-        else:
-            _r2 = {}
-        _res.update(_r1)
-        _res.update(_r2)
-        return _res
-
-    results = [__static(dfs, "基准")]
-
-    for values, dfg in dfs.groupby(keys):
-        if isinstance(values, str):
-            values = [values]
-        assert len(keys) == len(values)
-
-        name = "#".join([f"{key1}_{name1}" for key1, name1 in zip(keys, values)])
-        results.append(__static(dfg, name))
-    dfr = pd.DataFrame(results)
-    r_cols = [x for x in dfr.columns if x[-1] == 'b']
-    dfr[r_cols] = dfr[r_cols].round(2)
-    return dfr
-
-
 def discretizer(df: pd.DataFrame, col: str, n_bins=20, encode='ordinal', strategy='quantile'):
     """使用 KBinsDiscretizer 对连续变量在时间截面上进行离散化
 
     :param df: 数据对象
     :param col: 连续变量列名
     :param n_bins: 参见 KBinsDiscretizer 文档
+        https://scikit-learn.org/stable/modules/generated/sklearn.preprocessing.KBinsDiscretizer.html
     :param encode: 参见 KBinsDiscretizer 文档
     :param strategy: 参见 KBinsDiscretizer 文档
     :return:
@@ -93,9 +39,10 @@ def discretizer(df: pd.DataFrame, col: str, n_bins=20, encode='ordinal', strateg
 
     new_col = f'{col}_bins{n_bins}'
     results = []
-    for dt, dfg in tqdm(df.groupby('dt')):
+    for dt, dfg in tqdm(df.groupby('dt'), desc=f"{col}_bins{n_bins}"):
         kb = KBinsDiscretizer(n_bins=n_bins, encode=encode, strategy=strategy)
-        dfg[new_col] = kb.fit_transform(dfg[col].values.reshape(-1, 1)).ravel()
+        # 加1，使分组从1开始
+        dfg[new_col] = kb.fit_transform(dfg[col].values.reshape(-1, 1)).ravel() + 1
         results.append(dfg)
     df = pd.concat(results, ignore_index=True)
     return df
@@ -156,8 +103,6 @@ def generate_signals(bars: List[RawBar],
                      base_freq: AnyStr,
                      freqs: List[AnyStr],
                      get_signals: Callable,
-                     max_bi_count: int = 50,
-                     bi_min_len: int = 7,
                      signals_n: int = 0,
                      ):
     """获取历史信号
@@ -167,8 +112,6 @@ def generate_signals(bars: List[RawBar],
     :param base_freq: 合成K线的基础周期
     :param freqs: K线周期列表
     :param get_signals: 单级别信号计算函数
-    :param max_bi_count: 单个级别最大保存笔的数量
-    :param bi_min_len: 一笔最小无包含K线数量
     :param signals_n: 见 `CZSC` 对象
     :return: signals
     """
@@ -189,8 +132,7 @@ def generate_signals(bars: List[RawBar],
         bg.update(bar)
 
     signals = []
-    ct = CzscAdvancedTrader(bg, get_signals, max_bi_count=max_bi_count,
-                            bi_min_len=bi_min_len, signals_n=signals_n)
+    ct = CzscAdvancedTrader(bg, get_signals, signals_n=signals_n)
     for bar in tqdm(bars_right, desc=f'generate signals of {bg.symbol}'):
         ct.update(bar)
         signals.append(dict(ct.s))
@@ -479,7 +421,7 @@ def generate_stocks_signals(dc: TsDataCache,
 class SignalsPerformance:
     """信号表现分析"""
 
-    def __init__(self, dfs: pd.DataFrame, keys: List[AnyStr], dc: TsDataCache, base_freq="日线"):
+    def __init__(self, dfs: pd.DataFrame, keys: List[AnyStr], dc: TsDataCache = None, base_freq="日线"):
         """
 
         :param dfs: 信号表
@@ -682,22 +624,15 @@ class SignalsPerformance:
             '向前看时序': self.analyze_return('1b'),
             '向后看时序': self.analyze_return('1n'),
 
-            '信号数量与上证50相关性': self.analyze_corr_index('000016.SH'),
-            '信号数量与中证500相关性': self.analyze_corr_index('000905.SH'),
-            '信号数量与沪深300相关性': self.analyze_corr_index('000300.SH'),
             '信号数量与自身收益相关性': self.analyze_ar_counts(),
-
-            # 'b1b与自身收益相关性': self.analyze_b_bar('b1b'),
-            # 'b5b与自身收益相关性': self.analyze_b_bar('b5b'),
-            # 'b13b与自身收益相关性': self.analyze_b_bar('b13b'),
-            # 'b21b与自身收益相关性': self.analyze_b_bar('b21b'),
-            # 'b34b与自身收益相关性': self.analyze_b_bar('b34b'),
-            # 'b55b与自身收益相关性': self.analyze_b_bar('b55b'),
-            # 'b89b与自身收益相关性': self.analyze_b_bar('b89b'),
-            # 'b144b与自身收益相关性': self.analyze_b_bar('b144b'),
-            # 'b233b与自身收益相关性': self.analyze_b_bar('b233b'),
         }
 
+        if self.dc:
+            res.update({
+                '信号数量与上证50相关性': self.analyze_corr_index('000016.SH'),
+                '信号数量与中证500相关性': self.analyze_corr_index('000905.SH'),
+                '信号数量与沪深300相关性': self.analyze_corr_index('000300.SH'),
+            })
         if file_xlsx:
             writer = pd.ExcelWriter(file_xlsx)
             for sn, df_ in res.items():
