@@ -3,11 +3,12 @@
 author: zengbin93
 email: zeng_bin8888@163.com
 create_dt: 2021/11/21 17:48
-describe: 信号系统
+describe: 信号系统，注意：这里仅仅只是提供一些写信号的例子，用来做策略是不太行的
 """
 from . import bxt
 from . import ta
 from . import other
 from . import vol
 from . import cat
+from . import pos
 
